local _, addon = ...

-- functions
local format = string.format
local C_Timer = C_Timer
local C_Timer_NewTimer = C_Timer.NewTimer
local CreateFrame = CreateFrame
local GetContainerItemID = GetContainerItemID
local GetContainerItemLink = GetContainerItemLink
local GetContainerNumSlots = GetContainerNumSlots
-- constants
local NUM_BAG_SLOTS = NUM_BAG_SLOTS
-- frames
-- libs

local KEYSTONE_ITEM_ID = 138019

addon:RegisterCallback(addon.EVENT_BAG_SCAN_STARTED, function()
    addon:Debug("Scanning for keystones in bags...")
end)

addon:RegisterCallback(addon.EVENT_BAG_SCAN_FINISHED, function(_, bagId, slotId, link)
    addon:Debug(link and format("Found `%s`.", link) or "Could not find keystone in bags.")
    addon:SetCharacterKeystone(addon:GetCharacterKey(), link)
end)

addon:RegisterCallback(addon.EVENT_INITIALISED, function()
    local scanDelay = 3
    local scanTimer
    local function scanCallback()
        addon:Fire(addon.EVENT_BAG_SCAN_STARTED)

        for bagId = 0, NUM_BAG_SLOTS do
            for slotId = 0, GetContainerNumSlots(bagId) do
                local itemId = GetContainerItemID(bagId, slotId)
                if KEYSTONE_ITEM_ID == itemId then
                    local link = GetContainerItemLink(bagId, slotId)
                    addon:Fire(addon.EVENT_BAG_SCAN_FINISHED, bagId, slotId, link)
                    return
                end
            end
        end

        addon:Fire(addon.EVENT_BAG_SCAN_FINISHED)
    end

    local function onEvent()
        if scanTimer then
            scanTimer:Cancel()
        end

        scanTimer = C_Timer_NewTimer(scanDelay, scanCallback)
    end

    local f = CreateFrame("frame")
    f:SetScript("OnEvent", onEvent)
    f:RegisterEvent("BAG_UPDATE_DELAYED")
    f:RegisterEvent("CHALLENGE_MODE_COMPLETED")
    f:RegisterEvent("CHALLENGE_MODE_RESET")
    f:RegisterEvent("CHALLENGE_MODE_START")

    function addon:BagScannerOnEvent()
        onEvent()
    end

    onEvent()
end)
